import { ANALYZE_FOR_ENTRY_COMPONENTS, Directive, ElementRef, EventEmitter, Inject, InjectionToken, Input, NgModule, Output } from '@angular/core';
import { CommonModule } from '@angular/common';
var CounterUpConfig = (function () {
    /**
     * @param {?=} config
     */
    function CounterUpConfig(config) {
        if (config === void 0) { config = {}; }
        this.delay = 10;
        this.time = 100;
        this.beginAt = 0;
        Object.assign(this, config);
    }
    return CounterUpConfig;
}());
var CounterUpDirective = (function () {
    /**
     * @param {?} el
     * @param {?} config
     */
    function CounterUpDirective(el, config) {
        this.el = el;
        this.complete = new EventEmitter();
        this._pause = false;
        this._formatter = '';
        this._delay = config.delay;
        if (this._delay <= 0) {
            this._delay = 10;
        }
        this._time = config.time > 0 ? config.time : 100;
        this._beginAt = config.beginAt;
    }
    Object.defineProperty(CounterUpDirective.prototype, "number", {
        /**
         * @param {?} value
         * @return {?}
         */
        set: function (value) {
            if (typeof (value) === 'number') {
                value = value.toString();
            }
            this._isComma = /[0-9]+,[0-9]+/.test(value);
            value = value.replace(/,/g, '');
            // this._decimalPlaces = (value.split('.')[1] || []).length;
            if (this._beginAt > parseInt(value, 10)) {
                this._beginAt = parseInt(value, 10);
            }
            this._isTime = /[0-9]+:[0-9]+:[0-9]+/.test(value);
            // Convert time to total seconds
            if (this._isTime) {
                var /** @type {?} */ times = value.split(':');
                var /** @type {?} */ m = 1, /** @type {?} */ s = 0;
                while (times.length > 0) {
                    s += m * parseInt(times.pop(), 10);
                    m *= 60;
                }
                this._number = s;
            }
            else {
                this._number = parseInt(value, 10);
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CounterUpDirective.prototype, "offset", {
        /**
         * @param {?} value
         * @return {?}
         */
        set: function (value) {
            if (/^\d+$/.test(value)) {
                this._offset = parseInt(value, 10);
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CounterUpDirective.prototype, "formatter", {
        /**
         * @param {?} value
         * @return {?}
         */
        set: function (value) {
            this._formatter = value;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    CounterUpDirective.prototype.ngOnInit = function () {
        var /** @type {?} */ number, /** @type {?} */ type;
        if (this._isTime) {
            type = 'time';
        }
        else if (this._isComma) {
            type = 'comma';
        }
        else {
            type = 'text';
        }
        if (this.el.nativeElement.innerHTML.length === 0) {
            number = this._beginAt;
        }
        else {
            number = this[type + '2number'](this.el.nativeElement.innerHTML);
        }
        this._proceedNumber = number;
        this.el.nativeElement.innerHTML = this.format(this['number2' + type](number));
        this.start();
    };
    /**
     * @param {?} e
     * @return {?}
     */
    CounterUpDirective.prototype.ngOnChanges = function (e) {
        if (e.number && !e.number.firstChange) {
            this.start();
        }
    };
    /**
     * @return {?}
     */
    CounterUpDirective.prototype.ngOnDestroy = function () {
        if (this._timestamp) {
            clearTimeout(this._timestamp);
        }
    };
    /**
     * @return {?}
     */
    CounterUpDirective.prototype._increment = function () {
        var _this = this;
        if (this._timestamp) {
            clearTimeout(this._timestamp);
        }
        var /** @type {?} */ newValue = this._proceedNumber + this._offset, /** @type {?} */ text, /** @type {?} */ complete = false;
        if (newValue > this._number) {
            newValue = this._number;
            complete = true;
        }
        this._proceedNumber = newValue;
        if (this._isTime) {
            text = this.number2time(newValue);
        }
        else if (this._isComma) {
            text = this.number2comma(newValue);
        }
        else {
            text = this.number2text(newValue);
        }
        this.el.nativeElement.innerHTML = this.format(text);
        if (complete) {
            this.complete.emit({ target: this.el });
        }
        else if (!this._pause) {
            this._timestamp = setTimeout(function () { return _this._increment(); }, this._delay);
        }
    };
    /**
     * @return {?}
     */
    CounterUpDirective.prototype.start = function () {
        if (this._isTime) {
            this._offset = 1;
            this._delay = 1000;
        }
        else if (!this._offset) {
            this._offset = parseInt(((this._number - this._proceedNumber) / this._time).toString(), 10);
            if (this._offset < 1) {
                this._offset = 1;
            }
        }
        this._increment();
    };
    /**
     * @return {?}
     */
    CounterUpDirective.prototype.pause = function () {
        this._pause = true;
    };
    /**
     * @return {?}
     */
    CounterUpDirective.prototype.resume = function () {
        this._pause = false;
        this._increment();
    };
    /**
     * @param {?} text
     * @return {?}
     */
    CounterUpDirective.prototype.time2number = function (text) {
        text = /[0-9]+:[0-9]+:[0-9]+/.test(text) ? text : '00:00:00';
        var /** @type {?} */ times = text.split(':');
        var /** @type {?} */ m = 1, /** @type {?} */ number = 0;
        while (times.length > 0) {
            number += m * parseInt(times.pop(), 10);
            m *= 60;
        }
        return number;
    };
    /**
     * @param {?} number
     * @return {?}
     */
    CounterUpDirective.prototype.number2time = function (number) {
        var /** @type {?} */ hours = parseInt((number / 3600).toString(), 10) % 24, /** @type {?} */ minutes = parseInt((number / 60).toString(), 10) % 60, /** @type {?} */ seconds = number % 60;
        return (hours < 10 ? '0' + hours : hours) + ':' +
            (minutes < 10 ? '0' + minutes : minutes) + ':' +
            (seconds < 10 ? '0' + seconds : seconds);
    };
    /**
     * @param {?} text
     * @return {?}
     */
    CounterUpDirective.prototype.comma2number = function (text) {
        return parseInt(text.replace(/,/g, ''), 10);
    };
    /**
     * @param {?} number
     * @return {?}
     */
    CounterUpDirective.prototype.number2comma = function (number) {
        var /** @type {?} */ text = number.toString();
        while (/(\d+)(\d{3})/.test(text)) {
            text = text.replace(/(\d+)(\d{3})/, '$1' + ',' + '$2');
        }
        return text;
    };
    /**
     * @param {?} text
     * @return {?}
     */
    CounterUpDirective.prototype.text2number = function (text) {
        return parseInt(text, 10);
    };
    /**
     * @param {?} number
     * @return {?}
     */
    CounterUpDirective.prototype.number2text = function (number) {
        return number.toString();
    };
    /**
     * @param {?} text
     * @return {?}
     */
    CounterUpDirective.prototype.format = function (text) {
        if (this._formatter.length === 0) {
            return text;
        }
        return this._formatter.replace(/%s/g, text);
    };
    return CounterUpDirective;
}());
CounterUpDirective.decorators = [
    { type: Directive, args: [{
                selector: '[counter-up]'
            },] },
];
/**
 * @nocollapse
 */
CounterUpDirective.ctorParameters = function () { return [
    { type: ElementRef, },
    { type: CounterUpConfig, decorators: [{ type: Inject, args: [CounterUpConfig,] },] },
]; };
CounterUpDirective.propDecorators = {
    'number': [{ type: Input },],
    'offset': [{ type: Input },],
    'formatter': [{ type: Input },],
    'complete': [{ type: Output },],
};
var CounterUpOptions = new InjectionToken('config');
/**
 * @param {?} config
 * @return {?}
 */
function factoryCounterUpConfig(config) {
    return new CounterUpConfig(config);
}
var CounterUpModule = (function () {
    function CounterUpModule() {
    }
    /**
     * @param {?=} config
     * @return {?}
     */
    CounterUpModule.forRoot = function (config) {
        if (config === void 0) { config = {}; }
        return {
            ngModule: CounterUpModule,
            providers: [
                { provide: CounterUpOptions, useValue: config },
                { provide: ANALYZE_FOR_ENTRY_COMPONENTS, multi: true, useValue: config },
                { provide: CounterUpConfig, useFactory: factoryCounterUpConfig, deps: [CounterUpOptions] }
            ]
        };
    };
    return CounterUpModule;
}());
CounterUpModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                declarations: [CounterUpDirective],
                exports: [CounterUpDirective]
            },] },
];
/**
 * @nocollapse
 */
CounterUpModule.ctorParameters = function () { return []; };
/**
 * Generated bundle index. Do not edit.
 */
export { CounterUpOptions, factoryCounterUpConfig, CounterUpModule, CounterUpConfig, CounterUpDirective as ɵa };
//# sourceMappingURL=angular4-counter-up.es5.js.map
