import { ANALYZE_FOR_ENTRY_COMPONENTS, Directive, ElementRef, EventEmitter, Inject, InjectionToken, Input, NgModule, Output } from '@angular/core';
import { CommonModule } from '@angular/common';

class CounterUpConfig {
    /**
     * @param {?=} config
     */
    constructor(config = {}) {
        this.delay = 10;
        this.time = 100;
        this.beginAt = 0;
        Object.assign(this, config);
    }
}

class CounterUpDirective {
    /**
     * @param {?} el
     * @param {?} config
     */
    constructor(el, config) {
        this.el = el;
        this.complete = new EventEmitter();
        this._pause = false;
        this._formatter = '';
        this._delay = config.delay;
        if (this._delay <= 0) {
            this._delay = 10;
        }
        this._time = config.time > 0 ? config.time : 100;
        this._beginAt = config.beginAt;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set number(value) {
        if (typeof (value) === 'number') {
            value = value.toString();
        }
        this._isComma = /[0-9]+,[0-9]+/.test(value);
        value = value.replace(/,/g, '');
        // this._decimalPlaces = (value.split('.')[1] || []).length;
        if (this._beginAt > parseInt(value, 10)) {
            this._beginAt = parseInt(value, 10);
        }
        this._isTime = /[0-9]+:[0-9]+:[0-9]+/.test(value);
        // Convert time to total seconds
        if (this._isTime) {
            const /** @type {?} */ times = value.split(':');
            let /** @type {?} */ m = 1, /** @type {?} */ s = 0;
            while (times.length > 0) {
                s += m * parseInt(times.pop(), 10);
                m *= 60;
            }
            this._number = s;
        }
        else {
            this._number = parseInt(value, 10);
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set offset(value) {
        if (/^\d+$/.test(value)) {
            this._offset = parseInt(value, 10);
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set formatter(value) {
        this._formatter = value;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        let /** @type {?} */ number, /** @type {?} */ type;
        if (this._isTime) {
            type = 'time';
        }
        else if (this._isComma) {
            type = 'comma';
        }
        else {
            type = 'text';
        }
        if (this.el.nativeElement.innerHTML.length === 0) {
            number = this._beginAt;
        }
        else {
            number = this[type + '2number'](this.el.nativeElement.innerHTML);
        }
        this._proceedNumber = number;
        this.el.nativeElement.innerHTML = this.format(this['number2' + type](number));
        this.start();
    }
    /**
     * @param {?} e
     * @return {?}
     */
    ngOnChanges(e) {
        if (e.number && !e.number.firstChange) {
            this.start();
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this._timestamp) {
            clearTimeout(this._timestamp);
        }
    }
    /**
     * @return {?}
     */
    _increment() {
        if (this._timestamp) {
            clearTimeout(this._timestamp);
        }
        let /** @type {?} */ newValue = this._proceedNumber + this._offset, /** @type {?} */ text, /** @type {?} */ complete = false;
        if (newValue > this._number) {
            newValue = this._number;
            complete = true;
        }
        this._proceedNumber = newValue;
        if (this._isTime) {
            text = this.number2time(newValue);
        }
        else if (this._isComma) {
            text = this.number2comma(newValue);
        }
        else {
            text = this.number2text(newValue);
        }
        this.el.nativeElement.innerHTML = this.format(text);
        if (complete) {
            this.complete.emit({ target: this.el });
        }
        else if (!this._pause) {
            this._timestamp = setTimeout(() => this._increment(), this._delay);
        }
    }
    /**
     * @return {?}
     */
    start() {
        if (this._isTime) {
            this._offset = 1;
            this._delay = 1000;
        }
        else if (!this._offset) {
            this._offset = parseInt(((this._number - this._proceedNumber) / this._time).toString(), 10);
            if (this._offset < 1) {
                this._offset = 1;
            }
        }
        this._increment();
    }
    /**
     * @return {?}
     */
    pause() {
        this._pause = true;
    }
    /**
     * @return {?}
     */
    resume() {
        this._pause = false;
        this._increment();
    }
    /**
     * @param {?} text
     * @return {?}
     */
    time2number(text) {
        text = /[0-9]+:[0-9]+:[0-9]+/.test(text) ? text : '00:00:00';
        const /** @type {?} */ times = text.split(':');
        let /** @type {?} */ m = 1, /** @type {?} */ number = 0;
        while (times.length > 0) {
            number += m * parseInt(times.pop(), 10);
            m *= 60;
        }
        return number;
    }
    /**
     * @param {?} number
     * @return {?}
     */
    number2time(number) {
        const /** @type {?} */ hours = parseInt((number / 3600).toString(), 10) % 24, /** @type {?} */ minutes = parseInt((number / 60).toString(), 10) % 60, /** @type {?} */ seconds = number % 60;
        return (hours < 10 ? '0' + hours : hours) + ':' +
            (minutes < 10 ? '0' + minutes : minutes) + ':' +
            (seconds < 10 ? '0' + seconds : seconds);
    }
    /**
     * @param {?} text
     * @return {?}
     */
    comma2number(text) {
        return parseInt(text.replace(/,/g, ''), 10);
    }
    /**
     * @param {?} number
     * @return {?}
     */
    number2comma(number) {
        let /** @type {?} */ text = number.toString();
        while (/(\d+)(\d{3})/.test(text)) {
            text = text.replace(/(\d+)(\d{3})/, '$1' + ',' + '$2');
        }
        return text;
    }
    /**
     * @param {?} text
     * @return {?}
     */
    text2number(text) {
        return parseInt(text, 10);
    }
    /**
     * @param {?} number
     * @return {?}
     */
    number2text(number) {
        return number.toString();
    }
    /**
     * @param {?} text
     * @return {?}
     */
    format(text) {
        if (this._formatter.length === 0) {
            return text;
        }
        return this._formatter.replace(/%s/g, text);
    }
}
CounterUpDirective.decorators = [
    { type: Directive, args: [{
                selector: '[counter-up]'
            },] },
];
/**
 * @nocollapse
 */
CounterUpDirective.ctorParameters = () => [
    { type: ElementRef, },
    { type: CounterUpConfig, decorators: [{ type: Inject, args: [CounterUpConfig,] },] },
];
CounterUpDirective.propDecorators = {
    'number': [{ type: Input },],
    'offset': [{ type: Input },],
    'formatter': [{ type: Input },],
    'complete': [{ type: Output },],
};

const CounterUpOptions = new InjectionToken('config');
/**
 * @param {?} config
 * @return {?}
 */
function factoryCounterUpConfig(config) {
    return new CounterUpConfig(config);
}
class CounterUpModule {
    /**
     * @param {?=} config
     * @return {?}
     */
    static forRoot(config = {}) {
        return {
            ngModule: CounterUpModule,
            providers: [
                { provide: CounterUpOptions, useValue: config },
                { provide: ANALYZE_FOR_ENTRY_COMPONENTS, multi: true, useValue: config },
                { provide: CounterUpConfig, useFactory: factoryCounterUpConfig, deps: [CounterUpOptions] }
            ]
        };
    }
}
CounterUpModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                declarations: [CounterUpDirective],
                exports: [CounterUpDirective]
            },] },
];
/**
 * @nocollapse
 */
CounterUpModule.ctorParameters = () => [];

/**
 * Generated bundle index. Do not edit.
 */

export { CounterUpOptions, factoryCounterUpConfig, CounterUpModule, CounterUpConfig, CounterUpDirective as ɵa };
//# sourceMappingURL=angular4-counter-up.js.map
